#include "math/LogDouble.hh"
#include "bio/alphabet/Alphabet.hh"
#include "PairHMM2.hh"
#include "ParamPoint.hh"

class LikelihoodCalculator2 {
public:
	typedef math::LogDouble Prob;
	typedef PairHMM2::ProbMatrix ProbMatrix;
	typedef PairHMM2::ProbVector ProbVector;
	typedef PairHMM2::SeqPair SeqPair;
	typedef std::vector<SeqPair> SeqPairList;
	
	LikelihoodCalculator2(const bio::alphabet::Alphabet& alphabet);

	void setParams(const ParamPoint& params);
	void setParams(Prob mu, Prob delta, Prob epsilon);
	void getParams(Prob& mu, Prob& delta, Prob& epsilon) const;
	void getParams(double& mu, double& delta, double& epsilon) const;
	
	Prob getLikelihood(const std::string& s1,
					   const std::string& s2) const;

	Prob emIter(const std::string& s1,
				const std::string& s2);

	Prob emIter(const SeqPairList& seq_pairs);
	
	template<typename Gen>
	SeqPair generate(size_t len1, size_t len2, Gen& gen) const;
	
private:
	const bio::alphabet::Alphabet& alphabet;
	Prob mu;
	Prob delta;
	Prob epsilon;

	ProbMatrix hEmissions;
	ProbVector iEmissions;
	ProbVector dEmissions;
	ProbMatrix transitions;
};

template<typename Gen>
LikelihoodCalculator2::SeqPair
LikelihoodCalculator2::generate(size_t len1, size_t len2, Gen& gen) const {
	PairHMM2 phmm(hEmissions, iEmissions, dEmissions, transitions);
	SeqPair encoded(phmm.generate(len1, len2, gen));
	return SeqPair(alphabet.decode(encoded.first),
				   alphabet.decode(encoded.second));
}
